@@ -1,42 +1,10 @@
-#' Bayesian single-index regression with Gaussian process link and spike-and-slab prior
-#' @description Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n}
+#' Bayesian Single-Index Regression with Gaussian Process Link and Spike-and-Slab Prior
+#'
+#' @description Fits a single-index model \eqn{Y_i \sim \mathcal{N}(f(X_i'\theta), \sigma^2), i = 1,\cdots,n},
 #' where index vector \eqn{\theta} has a spike and slab prior and
 #' the link \eqn{f(\cdot)} is represented by Gaussian process and the
 #'
-#'
-#' @param x Numeric data.frame/matrix of predictors. Each row is an observation.
-#' @param y Numeric response numeric vector/matrix. Other types  are not available
-#' @param prior Optional named list of prior settings with sublists:
-#' \describe{
-#'     \item{\code{index}}{Spike and slab prior hyperparameters: Beta-binomial for variable selection (default \code{r1 = 1, r2 = 1}),
-#'     and normal distribution for selected variables (default: \eqn{N(0, \sigma_{\theta}^{2}}))}
-#'     \item{\code{link}}{Gaussian process prior hyperparameters \code{lambda}: Inverse-Gamma prior is assigned for \eqn{\lambda^{-1}}
-#'     (default \code{inv_lambda_shape = 1, inv_lambda_rate = 0.1})}
-#'     \item{\code{sigma2}}{Error-variance prior hyperparameters. An Inverse-Gamma prior is assigned to \eqn{\sigma^2}
-#'         where \code{shape} is shape parameter and \code{rate} is rate parameter of inverse gamma distribution.
-#'         (default \code{shape = 0.001, rate = 100})}
-#'   }
-#' @param init Optional named list of initial values. If the values are not assigned, they are randomly sampled from prior.
-#' \describe{
-#' \item{\code{index}}{
-#' \enumerate{
-#'      \item{\code{pi}: Initial selecting variable probability. (default: \code{0.5})}
-#'      \item{\code{nu}: Initial vector of inclusion indicators . By default, each \code{nu} is randomly drawn by  \eqn{Bernoulli(1/2)}}
-#'      \item{\code{index}: Initial vector of index. By default, each element of index vector, which is chosen by nu, is proposed by normal distribution.}
-#' }}
-#' \item{\code{link}}{Initial scalar of lambda (\code{inv_lambda}) for covariance of Gaussian process.}
-#' \item{\code{sigma2}}{Initial scalar error variance. (default: \code{0.01})}
-#' }
-#' @param sampling Logical. If \code{TRUE} (default), run MCMC; otherwise return prepared nimble model objects without sampling.
-#' @param fitted Logical. If \code{fitted = FALSE}, fitted values are not drawn and only \code{c("nu", "indexstar", "sigma2")} are monitored.
-#' If \code{fitted = TRUE} (default), fitted values drawn from posterior distribution are included in the output and \code{c("Xlin", "invlambda")} is additionally monitored for prediction.
-#' @param monitors2 Optional character vector of additional monitor nodes. To check the names of the nodes, set \code{fit <- gpSpike(x, y, sampling = FALSE)} and then inspect the variable names stored in the model object using \code{fit$model$getVarNames()}.
-#' @param niter Integer. Total MCMC iterations (default \code{10000}).
-#' @param nburnin Integer. Burn-in iterations (default \code{1000}).
-#' @param thin Integer. Thinning for monitors1 (default \code{1}).
-#' @param thin2 Integer. Optional thinning for \code{monitors2} (default \code{1}).
-#' @param nchain Integer. Number of MCMC chains (default \code{1}).
-#' @param setSeed Logical or numeric argument.  Further details are provided in \link[nimble]{runMCMC}.
+#' @inheritParams bsFisher
 #'
 #' @details
 #' \strong{Model} The single–index model is specified as \eqn{Y_i = f(X_i' \theta) + \epsilon_i},
@@ -52,11 +20,12 @@
 #'
 #' \strong{Priors}
 #' \itemize{
-#' \item Inclusion indicators \eqn{\nu_l}: Bernoulli(\eqn{\pi}).
-#'   \item Inclusion probability \eqn{\pi}: Beta(\eqn{r_1, r_2}).
-#'   \item Slab coefficients \eqn{\theta_l^*}: Gaussian \eqn{N(0, \sigma_\theta^2)}.
-#'   \item GP precision \eqn{\lambda^{-1}}: Gamma(\eqn{a_\lambda, b_\lambda}).
-#'   \item Error precision \eqn{(\sigma^2)^{-1}}: Gamma(\eqn{a_\sigma, b_\sigma}).
+#' \item The variable selection indicator \eqn{\nu} has Beta–Bernoulli hierarchy prior. Beta hyper-prior on the Bernoulli–inclusion probability \eqn{w},
+#'     inducing \eqn{p(\nu) \propto \mathrm{Beta}(r_1 + n_\nu, r_2 + p - n_\nu)} where  \eqn{n_\nu = \Sigma_{i=1}^{p}I(\nu_i = 1)}.
+#'     \eqn{r_1, r_2} are shape and rate parameter of beta distribution.
+#' \item Slab coefficients \eqn{\theta} have normal distribution with zero mean and \eqn{\sigma_\theta^2} variance.
+#'   \item GP precision \eqn{\lambda^{-1}} follows gamma distribution with shape parameter \eqn{a_\lambda}, and rate parameter \eqn{b_\lambda}.
+#'   \item Error precision \eqn{(\sigma^2)^{-1}} follows gamma distribution with shape parameter \eqn{a_\sigma}, and rate parameter \eqn{b_\sigma}.
 #' }
 #'
 #' \strong{Sampling} A random walk Metropolis algorithm is used to sample \eqn{\lambda^{-1}}
@@ -64,16 +33,36 @@
 #' The variance \eqn{\sigma^2} is directly sampled from posterior distribution.
 #' \eqn{f} is not directly sampled by MCMC.
 #'
-#'
-#' @return A \code{list} typically containing:
-#' \describe{
-#'   \item{\code{model}}{Nimble model}
-#'   \item{\code{sampler}}{Nimble sampler}
-#'   \item{\code{sampling}}{Posterior draws of \eqn{\nu}, \eqn{\theta^*}, \eqn{\sigma^2}, and nodes for fitted values by default. Variables specified in \code{monitors2} will be added if provided.}
-#'   \item{\code{fitted}}{If \code{fitted = TRUE}, in-sample fitted values.}
-#'   \item{\code{input}}{List of input values for prior, initial values and execution time without compiling.}
+#' \strong{Prior hyper-parameters}
+#' These are the prior hyper-parameters set in the function. You can define new values for each parameter in \link{prior_param}.
+#' \enumerate{
+#' \item Index vector: \code{index_nu_r1, index_nu_r2} gives the shape and rate parameter of beta-binomial prior, respectively.
+#'     For slab prior, normal distribution with zero mean is assigned for selected variables \eqn{\theta}. \code{index_sigma_theta} is for variance of \eqn{\theta}, and it is assigned 0.25 by default.
+#'     \item Link function: Inverse gamma prior is assigned for hyper-parameters \eqn{\lambda^{-1}}
+#'     `link_inv_lambda_shape` is shape parameter and `link_inv_lambda_rate` is rate parameter of inverse gamma distribution.
+#'     (default \code{link_inv_lambda_shape = 1, link_inv_lambda_rate = 0.1})
+#'     \item Error variance (\code{sigma2}): An Inverse gamma prior is assigned to \eqn{\sigma^2}
+#'         where \code{sigma2_shape} is shape parameter and \code{sigma2_rate} is rate parameter of inverse gamma distribution.
+#'         (default \code{sigma2_shape = 0.001, sigma2_rate = 100})
 #' }
 #'
+#' \strong{Initial values}
+#' These are the initial values set in the function. You can define new values for each initial value in \link{init_param}
+#' \enumerate{
+#' \item Index vector:
+#' \itemize{
+#'      \item{\code{index_pi}: Initial selecting variable probability. (default: \code{0.5})}
+#'      \item{\code{index_nu}: Initial vector of inclusion indicators . By default, each \code{index_nu} is randomly drawn by  \eqn{Bernoulli(1/2)}}
+#'      \item{\code{index}: Initial vector of index. By default, each element of index vector, which is chosen by indicator, is proposed by normal distribution.}
+#' }
+#' \item Link function: Initial scalar of lambda (\code{link_inv_lambda}) for covariance kernel of Gaussian process.
+#' \item Error variance (\code{sigma2}): Initial scalar error variance. (default: \code{0.01})
+#'
+#' }
+#'
+#'
+#'@inherit bsFisher return
+#'
 #' @examples
 #' \donttest{
 #' set.seed(123)
@@ -84,16 +73,23 @@
 #' X <- matrix(runif(n * d, -1, 1), nrow = n)
 #' index_vals <- as.vector(X %*% theta)
 #' y <- f(index_vals) + rnorm(n, 0, sigma)
+#' simdata <- data.frame(x = X, y = y)
+#' colnames(simdata) <- c(paste0("X", 1:4), "y")
 #'
 #' # One tool version
-#' fit <- gpSpike(X, as.vector(y))
+#' fit1 <- gpSpike(y ~ ., data = simdata,
+#'                niter = 5000, nburnin = 1000)
 #'
 #' # Split version
-#' models <- gpSpike(X, as.vector(y), sampling = FALSE)
+#' models <- gpSpike_setup(y ~ ., data = simdata)
 #' Ccompile <- compileModelAndMCMC(models)
-#' mcmc.out <- runMCMC(Ccompile$mcmc, niter = 5000, nburnin = 1000, thin = 1,
-#'                    nchains = 1, setSeed = TRUE, init = models$input$init,
-#'                    summary = TRUE, samplesAsCodaMCMC = TRUE)
+#' nimSampler <- get_sampler(Ccompile)
+#' initList <- getInit(models)
+#' mcmc.out <- runMCMC(nimSampler, niter = 5000, nburnin = 1000, thin = 1,
+#'                     nchains = 1, setSeed = TRUE, inits = initList,
+#'                     summary = TRUE, samplesAsCodaMCMC = TRUE)
+#' fit2 <- as_bsim(models, mcmc.out)
+#' summary(fit2)
 #' }
 #'
 #' @references
@@ -101,28 +97,51 @@
 #' Bayesian multiple index models for environmental mixtures.
 #' \emph{Biometrics}, 79(1), 462-474.
 #'
-#'
+#' @name gpSpike
 #' @export
-#'
+gpSpike <- function(formula, data,
+                    prior = NULL,
+                    init = NULL,
+                    monitors = NULL, niter = 10000, nburnin=1000,
+                    thin = 1, nchain = 1, setSeed = FALSE
+){
+  return(gpSpike.default(formula = formula, data = data,
+                         prior = prior,
+                         init = init,
+                         sampling = TRUE, monitors = monitors,
+                         niter = niter, nburnin=nburnin,
+                         thin = thin, nchain = nchain, setSeed = setSeed))
+}
 
+#' @rdname gpSpike
+#' @export
+gpSpike_setup <- function(formula, data,
+                          prior = NULL,
+                          init = NULL,
+                          monitors = NULL, niter = 10000, nburnin=1000,
+                          thin = 1, nchain = 1, setSeed = FALSE
+){
+  return(gpSpike.default(formula = formula, data = data,
+                         prior = prior,
+                         init = init,
+                         sampling = FALSE, monitors = monitors,
+                         niter = niter, nburnin=nburnin,
+                         thin = thin, nchain = nchain, setSeed = setSeed))
+}
 
-gpSpike <- function(x, y,
-                    prior = list(
-                            index = list(r1 = 1, r2 = 1, sigma_theta = 0.25),
-                            link = list(inv_lambda_shape = 1, inv_lambda_rate = 0.1),
-                            sigma2 = list(shape = 0.001, rate = 0.001)),
-                    init = list(index = list(pi = 0.5, nu = NULL, index = NULL),
-                                link = list(inv_lambda = NULL), sigma2 = NULL),
-                    sampling = TRUE, fitted = TRUE,
-                    monitors2 = NULL, niter = 10000, nburnin=1000,
-                    thin = 1, thin2 = NULL, nchain = 1, setSeed = FALSE
+gpSpike.default <- function(formula, data,
+                    prior = NULL,
+                    init = NULL,
+                    sampling = TRUE,
+                    monitors = NULL, niter = 10000, nburnin=1000,
+                    thin = 1, nchain = 1, setSeed = FALSE
 ){
   start1 <- Sys.time()
   index_raw <- 0; nu <- 0; invlambda <- 0; sigma2 <- 0
 
   # check sampling, prior, init parameters for independent execution
   checkOutput <- validate_and_finalize_args(
-    sampling, fitted, niter, nburnin, thin, thin2, nchain,
+    sampling, niter, nburnin, thin, nchain,
     prior, init, "spike", "gp"
   )
   prior <- checkOutput$priorlist_final
@@ -157,20 +176,47 @@ gpSpike <- function(x, y,
 
 
 
-  # check data dimension
-  if (!is.matrix(x) & !is.data.frame(x)){stop("x is not matrix/data.frame.")}
-  if (!is.vector(y) & !is.matrix(y)){stop("y is not vector or matrix.")}
-  if (is.matrix(y)){
-    if ((ncol(y) != 1)){
-      stop("y should be scalar vector or matrix.")
+  # Data
+  if (!is.data.frame(data)){
+    stop("data should be an data.frame.")
+  }
+
+  Call <- match.call()
+  indx <- match(c("formula","data"), names(Call), nomatch = 0L)
+  if (indx[1] == 0L)
+    stop("a 'formula' argument is required")
+  temp <- Call[c(1L,indx)]
+  temp[[1L]] <- quote(stats::model.frame)
+  m <- eval.parent(temp)
+  Terms <- attr(m,"terms")
+  formula <- as.character(formula)
+  response.name <- formula[2]
+  data.name <- strsplit(formula[3]," \\+ ")[[1]]
+  int.flag <- any(strsplit(formula[3]," \\* ")[[1]] == formula[3])
+  if(data.name[1]=="."){
+    tot.name <- response.name
+  } else{
+    tot.name <- c(response.name ,data.name)
+  }
+  if(!int.flag){
+    stop("BayesSIM cannot treat interaction terms")
+  }else if(!sum(duplicated(c(colnames(data),tot.name))[-c(1:ncol(data))])==
+           length(tot.name)){
+    stop(paste(paste(tot.name[duplicated(c(colnames(data),
+                                           tot.name))[-c(1:ncol(data))]],collapse=","),
+               " is/are not in your data"))
+  }else{
+    origY <- data[ ,response.name]
+    if(data.name[1]=="."){
+      origX <- data[,colnames(data) != response.name]
+    }else {
+      origX <- data[ ,data.name,drop=FALSE]
     }
   }
-  X <- as.matrix(x)
-  Y <- matrix(y, ncol = 1)
 
-  if (nrow(X) != nrow(Y)){
-    stop("x and y have different dimension.")
-  }
+  # X = origX, Y = origY
+  X <- as.matrix(origX)
+  Y <- as.matrix(origY)
 
   # data dimension
   N <- length(Y)
@@ -285,10 +331,11 @@ gpSpike <- function(x, y,
       index_raw[j] ~ dnorm(0, sd = sigma_theta)
       indexstar[j] <- index_raw[j] * nu[j]
     }
+    index[1:p] <- indexstar[1:p]/sqrt(sum(indexstar[1:p]^2))
 
     # Linear predictor
     for (i in 1:N){
-      Xlin[i] <- sum(X[i,1:p] * indexstar[1:p])
+      Xlin[i] <- sum(X[i,1:p] * index[1:p])
     }
 
     # likelihood
@@ -316,21 +363,13 @@ gpSpike <- function(x, y,
 
   # Assign samplers
   message("Assign samplers")
-  monitorsList <- c("nu", "indexstar", "sigma2")
-  if (fitted){
-    monitorsList <- c(monitorsList, "Xlin", "invlambda")
-  }
+  monitorsList <- c("nu", "index", "sigma2", "Xlin", "invlambda", "pi",
+                    "index_raw")
+
+  suppressMessages(mcmcConf <- configureMCMC(simpleModel,
+                                             monitors = monitorsList,
+                                             print = FALSE))
 
-  if (is.null(monitors2)){
-    suppressMessages(mcmcConf <- configureMCMC(simpleModel,
-                              monitors = monitorsList,
-                              print = FALSE))
-  } else{
-    suppressMessages(mcmcConf <- configureMCMC(simpleModel,
-                              monitors = monitorsList,
-                              monitors2 = monitors2,
-                              print = FALSE))
-  }
 
   # 1. invlambda
   mcmcConf$removeSamplers(c("invlambda"))
@@ -355,48 +394,36 @@ gpSpike <- function(x, y,
 
 
   mcmc1 <- buildMCMC(mcmcConf)
-  end1 <- Sys.time()
+
 
   if (!sampling){
     mcmc.out <- NULL
-    fittedResult <- NULL
-    sampMCMC <- NULL
+    samples <- NULL
 
   } else{
     # Compile
+    start2 <- Sys.time()
     message("Compile Model")
     suppressMessages(CsimpleModel <- compileNimble(simpleModel))
     message("Compile MCMC")
     suppressMessages(Cmcmc <- compileNimble(mcmc1, project = simpleModel,
-                           resetFunctions = TRUE))
+                                            resetFunctions = TRUE))
+    end2 <- Sys.time()
 
     # Sampling
-    start2 <- Sys.time()
     message("Run MCMC")
-    mcmc.out <- NULL
     if (setSeed == FALSE){
       seedNum <- setSeed
     }
-    if (is.null(monitors2)){
-      mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
-                          thin = thin,
-                          nchains = nchain, setSeed = seedNum,
-                          inits = inits_list,
-                          summary = FALSE, samplesAsCodaMCMC = TRUE)
-    } else{
-      mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
-                          thin = thin, thin2 = thin2,
-                          nchains = nchain, setSeed = seedNum,
-                          inits = inits_list,
-                          summary = FALSE, samplesAsCodaMCMC = TRUE)
-    }
+    mcmc.out <- runMCMC(Cmcmc, niter = niter, nburnin = nburnin,
+                        thin = thin,
+                        nchains = nchain, setSeed = seedNum,
+                        inits = inits_list,
+                        summary = FALSE, samplesAsCodaMCMC = TRUE)
+
+
     samples <- NULL
-    sampMCMC <- mcmc.out
-    # if (enableWAIC){
-    #   sampMCMC <- mcmc.out$samples
-    # } else{
-    #   sampMCMC <- mcmc.out
-    # }
+
     if (nchain > 1){
       for (i in 1:nchain){
         samples <- rbind(samples, mcmc.out[[i]])
@@ -404,66 +431,67 @@ gpSpike <- function(x, y,
     } else if (nchain == 1){
       samples <- mcmc.out
     }
-    end2 <- Sys.time()
-    start3 <- Sys.time()
 
-    # output
-    # fittedResult <- NULL
-    if (fitted){ # posterior fitted value output (mean, median, sd)
-      message("Compute posterior fitted value")
+    # # standardize index (make length = 1)
+    # namesIndex <- paste0("indexstar", 1:p)
+    # samples[, namesIndex] <- t(apply(samples[, namesIndex, drop = FALSE], 1, function(x) {
+    #   x / sqrt(sum(x^2))
+    # }))
 
-      nsamp <- nrow(samples)
-      namesXlin <- paste0("Xlin[", 1:N, "]")
-      XlinSample <- samples[, namesXlin]
-      sigma2_samples <- samples[, "sigma2"]
-      namesIndex <- paste0("indexstar[", 1:p, "]")
-      indexstarSample <- samples[, namesIndex]
-      invlambdaSample <- samples[,"invlambda"]
-      newdataMat <- scaleX
 
-      message("Compile function..")
-      suppressMessages(cpred_gpSpike <- compileNimble(pred_gpSpike))
-      message("Computing predicted value..")
-      fittedValue <- cpred_gpSpike(newdataMat, nsamp, as.vector(Y), indexstarSample,
-                                XlinSample, sigma2_samples, invlambdaSample)
-
-      # fittedResult <- data.frame(mean = apply(fittedValue, 2, mean),
-      #                            median = apply(fittedValue, 2, median),
-      #                            sd = apply(fittedValue, 2, sd),
-      #                            LB = apply(fittedValue, 2, quantile, probs = 0.025, na.rm = TRUE),
-      #                            UB = apply(fittedValue, 2, quantile, probs = 0.975, na.rm = TRUE))
-
-      fittedResult <- fittedValue
-    } else{
-      fittedResult <- NULL
-    }
   }
-  end3 <- Sys.time()
+  end1 <- Sys.time()
 
+  ## Input options
   ## Input options
   if (!sampling){
     time <- NULL
-  } else if (!fitted){
-    samp_time <- difftime(end2, start2, units = "secs") + difftime(end1, start1, units = "secs")
-    time <- list(samp = samp_time)
   } else{
-    samp_time <- difftime(end2, start2, units = "secs") + difftime(end1, start1, units = "secs")
-    fitted_time <- difftime(end3, start3, units = "secs")
-    time <- list(samp = samp_time, fitted = fitted_time)
+    time <- difftime(end1, start1, units = "secs") - difftime(end2, start2, units = "secs")
   }
-  inputOptions <- list(data = list(x = X, y = Y),
+
+
+  # results ---------------------------------------------------------------------
+  #Model point estimation
+  modelESTLIST <- bsimFit_pointest(samples, X, Y)
+
+  inputOptions <- list(origdata = list(x = X, y = Y), formula = formula,
                        prior = list(index = list(r1 = a0, r2 = b0, sigma_theta = sigma_theta),
                                     link = list(inv_lambda_shape = a_lam, inv_lambda_rate = b_lam),
                                     sigma2 = list(shape = a_sig, rate = b_sig)),
-                       init = inits_list,
+                       samplingOptions = list(sampling = sampling,
+                                              monitors = monitors, niter = niter,
+                                              nburnin = nburnin, thin = thin,
+                                              nchain = nchain, setSeed = setSeed),
+                       init = inits_list, # initial value for MCMC
                        time = time)
 
 
+  if (sampling){
+    out <- list(coefficients = modelESTLIST$coefficients,
+                ses_coef = modelESTLIST$ses_coef, se = modelESTLIST$se,
+                residuals = modelESTLIST$residuals,
+                fitted.values = modelESTLIST$fitted.values,
+                linear.predictors = modelESTLIST$linear.predictors,
+                gof = modelESTLIST$gof,
+                samples = mcmc.out, # return of runMCMC
+                input = inputOptions,
+                defModel = simpleModel, defSampler = mcmc1,
+                modelName = "gpSpike")
 
-  out <- list(model = simpleModel, sampler = mcmc1, sampling = sampMCMC,
-              fitted = fittedResult, input = inputOptions,
-              modelName = "gpSpike")
-  class(out) = "bsimGp"
+    class(out) = "bsim"
+
+
+  } else{
+    out <- list(input = inputOptions,
+                defModel = simpleModel,
+                defSampler = mcmc1,
+                modelName = "gpSpike")
+
+    class(out) = "bsimSetup"
+
+  }
   return(out)
+
 }
 
